#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

class ActiveRecord::Base
  class << self
    # Disable notification of observers during the execution of the block passed
    # to the method.
    def disable_observers(flag = true)
      if block_given?
        begin
          @observers_disabled = true
          yield
        ensure
          @observers_disabled = false
        end
      else
        @observers_disabled = flag != false
      end
    end

    def enable_observers
      @observers_disabled = false
    end

    # Returns true if the observers are currently disabled for this class
    # or its ancestors
    def observers_disabled?
      @observers_disabled || 
        superclass.respond_to?(:observers_disabled?) && superclass.observers_disabled?
    end
  end

  private
  def notify(method) #:nodoc:
    return if self.class.observers_disabled?
    self.class.changed
    self.class.notify_observers(method, self)
  end
end
