#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

require File.expand_path(File.dirname(__FILE__) + "/../../spec_helper")

class Common < RailsUtils::Model::Tableless
end

class Blog < Common
  column :published, :boolean
end

class BlogObserver < ActiveRecord::Observer
  def after_save(blog)
    blog.published = true
  end
end

class Comment < Common
  column :published, :boolean
end

class CommentObserver < ActiveRecord::Observer
  def after_save(comment)
    comment.published = true
  end
end

BlogObserver.instance
CommentObserver.instance

describe ActiveRecord::Base, "#disable_observers" do
  it "should prevent observers from being called" do
    Blog.create(:published => false).published.should == true
    Blog.disable_observers
    Blog.create(:published => false).published.should == false    
    Blog.enable_observers
    Blog.create(:published => false).published.should == true
  end

  it "should prevent observers from being called during execution of the block" do
    Blog.create(:published => false).published.should == true
    Blog.disable_observers do
      Blog.create(:published => false).published.should == false
    end
    Blog.create(:published => false).published.should == true
  end

  it "should allow other observers to still be active" do
    Blog.disable_observers do
      Blog.create(:published => false).published.should == false
      Comment.create(:published => false).published.should == true
    end
  end

  it "should allow multiple classes' observers to be disabled by disabling at a common superclass" do
    Common.disable_observers do
      Blog.create(:published => false).published.should == false
      Comment.create(:published => false).published.should == false
    end
  end
end
